#include <at89S8252.h>
#include <string.h>
/*-------------------------------------------------------------------
 Name/Nome: lcdgrafc.c
 Version/Versao: 1.0 alpha
 Description/Descricao:
    ROTINAS EM C PARA LCD GRAFICO COM CONTROLADOR KS0108
    (lcd grafico 128x64)
 Author/Autor: Rafael B. Ferrari
 Email: rafaelbf@hotmail.com
-------------------------------------------------------------------*/

// ponto inicial eh 1,1 e naum 0,0
// otimizar lcdbox
// terminar Elipse
// ta ocupando muita RAm

// Altere as definicoes dos pinos ligados no LCD



#include "lcdfonts.c"

// Hardware
#define LCD_DI      P2_0    
#define LCD_RW      P2_1    
#define LCD_EN      P2_2    
#define LCD_CS1     P2_3
#define LCD_CS2     P2_4
#define LCD_RST     P2_5
#define LCD_DATA    P0              
// Registradores do Lcd
#define LCD_X_ADDR      0x0B8       //Adress base for Page X0 / Endereco base da pagina 0
#define LCD_Y_ADDR      0x40        //Adress base for Y0 / Endereco base da coluna 0
#define LCD_START_LINE  0x0C0       //Adress base for Z0 line / Endereco base da linha 0
#define LCD_ON          0x3F        //Turn display on / Instrucao para ligar o display
#define LCD_OFF         0x3E        //Turn display off / Instrucao para desligar o display

#define LCD_RIGHT       0
#define LCD_LEFT        1

#define     MAXBUF          10          // tamanho maximo do buffer de caracteres


#define LCDInst(d)  _lcdwrite(d,0)
#define LCDData(d)  _lcdwrite(d,1)

#define LCDClear()  LCDFill(0)


unsigned char        tmp;                            // Temporario


void Inicializa(void);
void LCDSetDot(unsigned char Xaxis, unsigned char Yaxis, unsigned char point);
unsigned char LCDGetDot(unsigned char Xaxis, unsigned char Yaxis);
void LCDLine(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char Yaxis2, unsigned char pmode);
void LCDBox(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char u8Yaxis2, unsigned char type, unsigned char pmode);
void LCDFillRect(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char Yaxis2, unsigned char pmode);
void LCDDrawText(unsigned char *text, unsigned char Xaxis, unsigned char Yaxis, unsigned char *font,unsigned char spc, unsigned char pmode);
void LCDDrawImg(unsigned char Xaxis, unsigned char Yaxis, unsigned char *img);
void LCDFill(unsigned char data);
void LCDInit(void);
void LCDSide(unsigned char side);
void _lcddelay(void);
void _lcdwait(void);
void _lcdwrite(unsigned char wdata, unsigned char di);
unsigned char _lcdread(void);
void            Atraso(unsigned char ms);
void            DelSeg(unsigned char secs);




void main(void)
{

    Inicializa();   // inicializa 8051, verifique nessa rotina se precisa configurar mais algum periferico
    LCDInit();    // inicializa lcd
    

  while(1)
  {
    LCDBox(1,6,55,26,1,1);
    LCDDrawText("teste",18,9,&FONT8x8,1,1); // 5 parametro=espacamento / 6 parametro=modo do fundo (0 ou 1)
    LCDDrawText("teste2",80,12,&FONT5x7,1,1);
    DelSeg(1);  // atraso 1 seg
    LCDClear(); // limpa lcd
  }




}




/*    
*****************************************************************************************
************************ Inicializacao do 8051 e perifericos ****************************
*/
void Inicializa(void)
{

    tmp = 0;

    
    SCON = 0x50;            // Serial modo 1 (8 bits)
    T2CON = 0x34;           // Configuracao do Timer 2
//    PS = 1;
    RCAP2H = 0xFF;
//    RCAP2L = 0xD9;          // 19200 bps com cristal de 24Mhz
    RCAP2L = 0xEE;          // 19200 bps com cristal de 11,???Mhz

    
//    SCON = 0x50;            // 01010000 // Serial no modo 1 (uart 8 bits de dados)
//    TMOD = 0x21;            // 00100001 // Timer1 modo 2 (8bits para a serial) & Timer0 modo 1 (16bits)
//    PCON = 128;               // Smod ligado
    TMOD = 1;               // Timer0 modo 1
    IE = 0x90;               // Liga interrupcao da serial
//    TH1 = VELOSERIAL;       // Define a velocidade da serial
//    TR1 = 1;                // Liga o timer da serial
    TR0 = 0;                // Desliga o timer 0
    TR2 = 1;

}
/*
*****************************************************************************************
*/


void LCDSetDot(unsigned char Xaxis, unsigned char Yaxis, unsigned char point)
{
unsigned char DataRead=0;
unsigned char column=0;
  
//    LCDInst(LCD_START_LINE); /* Set adress for line 0 */

if ((Xaxis < 128) && (Yaxis < 64))
{
    if (Xaxis < 64)
    {
        LCDSide(LCD_LEFT);    /* Select left side */
        column = LCD_Y_ADDR + Xaxis;
    }
    else 
    {
        LCDSide(LCD_RIGHT);    /* Select left side */         
        column = LCD_Y_ADDR + Xaxis - 64;
    }

    LCDInst(LCD_X_ADDR + (Yaxis / 8)); /* Select page number */       
    LCDInst(column);  /* Select column */            

//if (point != 2)
//{
    DataRead = _lcdread();                    /* read the current location */
    DataRead = _lcdread();                    /* on the LCD. dummy read */   
                                                                                     
    LCDInst(LCD_X_ADDR + (Yaxis / 8)); /* Select page number */       
    LCDInst(column);  /* Select column */            
//}

if (point == 0)
    LCDData(DataRead & (255 - (1 << (Yaxis % 8))));
else if (point > 0)
    LCDData(DataRead | (1 << (Yaxis % 8)));


//    LCDInst(LCD_START_LINE); /* Set adress for line 0 */
}
}


unsigned char LCDGetDot(unsigned char Xaxis, unsigned char Yaxis)
{
unsigned char DataRead=0;
unsigned char column=0;
  

if ((Xaxis < 128) && (Yaxis < 64))
{
    if (Xaxis < 64)
    {
        LCDSide(LCD_LEFT);    /* Select left side */
        column = LCD_Y_ADDR + Xaxis;
    }
    else 
    {
        LCDSide(LCD_RIGHT);    /* Select left side */         
        column = LCD_Y_ADDR + Xaxis - 64;
    }

    LCDInst(LCD_X_ADDR + (Yaxis / 8)); /* Select page number */       
    LCDInst(column);  /* Select column */            
                                                                                     
    DataRead = _lcdread();                    /* read the current location */
    DataRead = _lcdread();                    /* on the LCD. dummy read */   

DataRead = ((DataRead >> (Yaxis % 8)) & 1);

                                                                                     
//    LCDInst(LCD_X_ADDR + (Yaxis / 8)); /* Select page number */       
//    LCDInst(column);  /* Select column */            
//if (point > 0)
//    LCDData(DataRead | (1 << (Yaxis % 8)));/* plot the dot */   
//else
//    LCDData(DataRead & (255 - (1 << (Yaxis % 8))));

}
return DataRead;
}


// rotina para tracar linhas utilizando algoritmo de Breseham
void LCDLine(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char Yaxis2, unsigned char pmode)
{
unsigned long dx,dy;
signed char stepx,stepy;
signed long frac;

//Xaxis1--;
//Xaxis2--;
//Yaxis1--;
//Yaxis2--;

if (Xaxis1 > Xaxis2)
{
	dx = (Xaxis1 - Xaxis2);
	stepx = -1;
}
else
{
	dx = (Xaxis2 - Xaxis1);
	stepx=1;
}
if (Yaxis1 > Yaxis2)
{
	dy = (Yaxis1 - Yaxis2);
	stepy=-1;
}
else
{
	dy = (Yaxis2 - Yaxis1);
	stepy=1;
}
dx <<= 1;
dy <<= 1;

	LCDSetDot(Xaxis1,Yaxis1,pmode);
	if (dx > dy)
	{
		frac = dy - (dx >> 1);
		while(Xaxis1 != Xaxis2)
		{
			if (frac >= 0)
			{
				Yaxis1 += stepy;
				frac -= dx;
			}
			Xaxis1 += stepx;
			frac += dy;
			LCDSetDot(Xaxis1,Yaxis1,pmode);
		}
	}
	else
	{
		frac = dx - (dy >> 1);
		while(Yaxis1 != Yaxis2)
		{
			if (frac >= 0)
			{
				Xaxis1 += stepx;
				frac -= dy;
			}
			Yaxis1 += stepy;
			frac += dx;
			LCDSetDot(Xaxis1,Yaxis1,pmode);
		}
	}


}


void LCDBox(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char Yaxis2, unsigned char type, unsigned char pmode)
{
unsigned char CurrentValue=0;


if (type == 0)
{
 /* Draw the two horizontal lines */
    for (CurrentValue = 0; CurrentValue < Xaxis2 - Xaxis1+ 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis1, pmode);
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis2, pmode);
    }
   
   /* draw the two vertical lines */
    for (CurrentValue = 0; CurrentValue < Yaxis2 - Yaxis1 + 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1, Yaxis1 + CurrentValue, pmode);
        LCDSetDot(Xaxis2, Yaxis1 + CurrentValue, pmode);
    }
}
else if (type == 1)
{
    for (CurrentValue = 2; CurrentValue < Xaxis2 - Xaxis1- 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis1, pmode);
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis2, pmode);
    }
   
    for (CurrentValue = 2; CurrentValue < Yaxis2 - Yaxis1 - 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1, Yaxis1 + CurrentValue, pmode);
        LCDSetDot(Xaxis2, Yaxis1 + CurrentValue, pmode);
    }
    
        LCDSetDot(Xaxis1+1,Yaxis1+1,pmode);
        LCDSetDot(Xaxis1+1,Yaxis2-1,pmode);
        LCDSetDot(Xaxis2-1,Yaxis1+1,pmode);
        LCDSetDot(Xaxis2-1,Yaxis2-1,pmode);
}
else if (type == 3)
{
    for (CurrentValue = 2; CurrentValue < Xaxis2 - Xaxis1- 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis1, pmode);
        LCDSetDot(Xaxis1 + CurrentValue, Yaxis2, pmode);
    }
   
    for (CurrentValue = 2; CurrentValue < Yaxis2 - Yaxis1 - 1; CurrentValue++) 
    {
        LCDSetDot(Xaxis1, Yaxis1 + CurrentValue, pmode);
        LCDSetDot(Xaxis2, Yaxis1 + CurrentValue, pmode);
    }
    
        LCDSetDot(Xaxis1+1,Yaxis1+1,pmode);
        LCDSetDot(Xaxis1+1,Yaxis2-1,pmode);
        LCDSetDot(Xaxis2-1,Yaxis1+1,pmode);
        LCDSetDot(Xaxis2-1,Yaxis2-1,pmode);
        
        LCDFillRect(Xaxis1, Yaxis1, Xaxis2, Yaxis2,1);
}



//    if (type > 0)
//    {
//        LCDSetDot(u8Xaxis1,u8Yaxis1,0);
//        LCDSetDot(u8Xaxis1,u8Yaxis2,0);
//        LCDSetDot(u8Xaxis2,u8Yaxis1,0);
//        LCDSetDot(u8Xaxis2,u8Yaxis2,0);
//    }
//    if (type ==2)
//    {
//        LCDBox(u8Xaxis1 +1, u8Yaxis1+1,u8Xaxis2-1,u8Yaxis2-1,0,pmode);
//    }

}

/*
void LCDElipse(unsigned char Xaxis, unsigned char Yaxis, unsigned char radx, unsigned char rady, unsigned char quads, unsigned char pmode)
{
int p1;
int tmpx,tmpy;

tmpx =0;
tmpy =0;

LCDSetDot(Xaxis, Yaxis + rady,pmode);
LCDSetDot(Xaxis-0, Yaxis + rady,pmode);
LCDSetDot(Xaxis-0, Yaxis - rady,pmode);
LCDSetDot(Xaxis, Yaxis - rady,pmode);

p1 = (rady^2) - (radx^2 * rady) + (radx^2 / 4);
while(rady^2 * tmpx >= radx^2 * tmpy)
{
tmpx += 1;
if (p1 < 0) p1 = p1 + (2 * (rady^2) * tmpx) + rady^2;
else
{
    tmpy -= 1;
    p1 = p1 + (2 * rady^2 * tmpx) - (2 * radx^2 * tmpy) + rady^2;
}
LCDSetDot(Xaxis + tmpx, Yaxis + tmpy,pmode);
LCDSetDot(Xaxis - tmpx, Yaxis + tmpy,pmode);
LCDSetDot(Xaxis - tmpx, Yaxis - tmpy,pmode);
LCDSetDot(Xaxis + tmpx, Yaxis - tmpy,pmode);
}


p1 = (rady^2 * tmpx) + (radx^2 * (tmpy-1)^2) - radx^2 * rady^2;
tmpy -= 1;
if (p1 >= 0) p1 -= 2 * radx^2 * tmpy + radx^2;
else
{
    tmpx += 1;
    p1 += 2 * rady^2 * tmpx - 2 * radx^2 * tmpy + rady^2;
}
LCDSetDot(Xaxis + tmpx, Yaxis + tmpy,pmode);
LCDSetDot(Xaxis - tmpx, Yaxis + tmpy,pmode);
LCDSetDot(Xaxis - tmpx, Yaxis - tmpy,pmode);
LCDSetDot(Xaxis + tmpx, Yaxis - tmpy,pmode);

}
*/
/*
void LCDCircle(unsigned char Xaxis, unsigned char Yaxis, unsigned char radi, unsigned char octs, unsigned char pmode)
{
int tmpx,tmpy;
int p;

    tmpx = 0;
    tmpy = radi;
    p = 1 - radi;
    while(tmpx <= tmpy)
    {
        if (octs & 1) LCDSetDot(Xaxis + tmpy, Yaxis - tmpx, pmode);        // 1
        if (octs & 2) LCDSetDot(Xaxis + tmpx, Yaxis - tmpy, pmode);        // 2
        if (octs & 4) LCDSetDot(Xaxis - tmpx, Yaxis - tmpy, pmode);        // 3
        if (octs & 8) LCDSetDot(Xaxis - tmpy, Yaxis - tmpx, pmode);        // 4
        if (octs & 16) LCDSetDot(Xaxis - tmpy, Yaxis + tmpx, pmode);       // 5
        if (octs & 32) LCDSetDot(Xaxis - tmpx, Yaxis + tmpy, pmode);       // 6
        if (octs & 64) LCDSetDot(Xaxis + tmpx, Yaxis + tmpy, pmode);       // 7
        if (octs & 128) LCDSetDot(Xaxis + tmpy, Yaxis + tmpx, pmode);      // 8

        tmpx += 1;
        if (p < 0)
            p += (2*tmpx) + 1;
        else
        {
            tmpy -= 1;
            p += (2*tmpx) + 1 - (2*tmpy);
        }
    }
}
*/
// Preenchimento de areas com algoritmo linha de varredura
void LCDFillRect(unsigned char Xaxis1,unsigned char Yaxis1,unsigned char Xaxis2,unsigned char Yaxis2, unsigned char pmode)
{
/*
determinar xmin, xmax, ymin, ymax
para L de ymin+1 at ymax-1 passo 1
flag = 0
para C de xmin at xmax passo 1
se leia_cor_do_pixel(C, L) = cor_da_borda e
leia_cor_do_pixel(C+1, L) = cor_do_fundo e
flag = 0 ento
flag = 1
seno
se leia_cor_do_pixel(c, l) = cor_da_borda e
leia_cor_do_pixel(c+1, l) = cor_do_fundo e
flag = 1 ento
flag = 0
fim se
fim se
se leia_cor_do_pixel(C, L) = cor_do_fundo e
flag = 1 ento
acende_pixel(C, L, cor_de_preenchimento)
fim se
prximo C
prximo L
*/  
unsigned char fl, col, line;

for(line=Yaxis1+1; line != Yaxis2; line++)
{
    fl = 0;
    for(col=Xaxis1; col <= Xaxis2; col++)
    {
        if ((LCDGetDot(col, line) == 1) && (LCDGetDot(col+1, line) == 0) && (fl == 0))
        fl = 1;
        else
        {
            if ((LCDGetDot(col, line) == 1) && (LCDGetDot(col+1, line) == 0) && (fl == 1))
            fl = 0;
        }
        if ((LCDGetDot(col, line) == 0) && (fl == 1))
        LCDSetDot(col, line,1);

    }
}
    
}

/*
void LCDDrawChar(unsigned char chr, unsigned char Xaxis, unsigned char Yaxis, unsigned char *font)
{
unsigned int col;
unsigned char bits;
//unsigned char chr;

/////// desatualizado

    LCDSide(LCD_LEFT);
    LCDInst(LCD_X_ADDR | 0);
    LCDInst(LCD_Y_ADDR);
    for(col=(chr*(font[0]))+2;col<(chr*(font[0]))+2+font[0];col++)
    {
//        if (Xaxis > 127)
//        {
//            Xaxis = 0;
//            Yaxis += font[0];
//        }

        // desenha um caractere, bit a bit
        for(bits=0;bits<font[1];bits++)
            if ((font[col] >> bits) & 1)
                LCDSetDot(Xaxis,Yaxis+bits,1);

    }

}
*/

// spc eh o espacamento, pmode eh o modo de desenho dos pontos
void LCDDrawText(unsigned char *text, unsigned char Xaxis, unsigned char Yaxis, unsigned char *font, unsigned char spc, unsigned char pmode)
{
unsigned int col;
unsigned char bits;
unsigned char chr;

    LCDSide(LCD_LEFT);   /* Select left side */
    LCDInst(LCD_X_ADDR | 0); /* Set the page number */
    LCDInst(LCD_Y_ADDR);  /* Set column to 0 */
for(chr=0;chr<strlen(text);chr++)
{
    for(col=((text[chr]-font[2])*(font[0]))+3;col<((text[chr]-font[2])*(font[0]))+3+font[0];col++)
    {
        if (Xaxis > 127)
        {
            Xaxis = 0;
            Yaxis += font[1];
        }
        for(bits=0;bits<=font[1];bits++)                 // desenha uma coluna bit a bit
            if ((font[col] >> bits) & 1)
                LCDSetDot(Xaxis,Yaxis+bits,pmode);
            Xaxis++;

    }
    for(bits=0;bits!=spc;bits++)                        // coloca espacamento de caracteres
        Xaxis++;
}

}

// para converter uma imagem use o conversor de BMP para C do site 8052.com
// inclua i arquivo gerado com um #include e chame a rotina de maneira como a LCDDrawText
void LCDDrawImg(unsigned char Xaxis, unsigned char Yaxis, unsigned char *img)
{
unsigned char col,pag;
Yaxis--;
    LCDSide(LCD_LEFT);
    LCDInst(LCD_X_ADDR | 0);
    LCDInst(LCD_Y_ADDR + Yaxis);

    for(pag=0;pag<img[0];pag++)
    {

        LCDSide(LCD_LEFT);
        LCDInst(LCD_Y_ADDR + Yaxis);
        LCDInst(LCD_X_ADDR | pag);
        
        for(col=0;col<img[1];col++)
        {
            if ((col + Yaxis) == 64)
            {
                    LCDSide(LCD_RIGHT);
                    LCDInst(LCD_Y_ADDR);
                    LCDInst(LCD_X_ADDR | pag);
            }
            LCDData(img[(col*img[0])+1+(img[0]-pag)]);
        }


    }
}

// limpa / preenche o LCD inteiro com um padrao
void LCDFill(unsigned char dado)
{
unsigned char Page=0;
unsigned char Column=0;
    
    for (Page = 0; Page < 8; Page++) 
    {
        LCDSide(LCD_LEFT);   /* Select left side */
        LCDInst(LCD_X_ADDR | Page); /* Set the page number */
        LCDInst(LCD_Y_ADDR);  /* Set column to 0 */
        
        for (Column = 0; Column <= 63; Column++) 
            LCDData(dado);

        LCDSide(LCD_RIGHT); /* Select right side */
        LCDInst(LCD_X_ADDR | Page); /* Set the page number */
        LCDInst(LCD_Y_ADDR); /* Set column to 0 */

        for (Column = 64; Column <= 127; Column++) 
            LCDData(dado);

     }
}



void LCDInit(void)
{
    LCD_DATA = 0;
    LCD_DI = 0;
    LCD_RW = 0;
    LCD_EN = 0;
    LCD_CS1 = 0;
    LCD_CS2 = 0;
   
    LCD_RST = 1;
    _lcddelay();
    LCD_RST=0;
    _lcddelay();
    _lcddelay();
    LCD_RST=1;

    LCDSide(LCD_LEFT);
    LCDInst(LCD_OFF); /* Display OFF */
    LCDInst(LCD_START_LINE);
    LCDInst(LCD_X_ADDR);
    LCDInst(LCD_Y_ADDR);
    LCDInst(LCD_ON); /* Display ON */
   
    LCDSide(LCD_RIGHT);
    LCDInst(LCD_OFF); /* Display OFF */
    LCDInst(LCD_START_LINE);
    LCDInst(LCD_X_ADDR);
    LCDInst(LCD_Y_ADDR);
    LCDInst(LCD_ON); /* Display ON */
    
    LCDClear();
}

void LCDSide(unsigned char side)
{
    if(side == LCD_RIGHT)
    {
//        LCD_DI=0;
//        LCD_EN=0;
//        LCD_RW=1;
        LCD_CS1=0;  
        LCD_CS2=1;
//        LCDInst(LCD_Y_ADDR); /* Set column to 0 */
    }
    else
    {
//        LCD_DI=0;
//        LCD_EN=0;
//        LCD_RW=1;
        LCD_CS1=1;  
        LCD_CS2=0;
//        LCDInst(LCD_Y_ADDR); /* Set column to 0 */
    }
}



void _lcddelay(void)
{
//unsigned int x;
//    for(x=0;x<=22;x++);
}


void _lcdwait(void)
{
unsigned char lcddata;
    LCD_DI = 0;        /* Instruction mode */
    LCD_RW = 1;        /* Read mode */
    LCD_DATA=0xFF;  /* set LCD_DATA port in input mode */
//    while((lcddata & 0x90) > 0)
//    {
//        LCD_EN  = 1;
//        _lcddelay();
//        lcddata = LCD_DATA;
//        LCD_EN  = 0;
//    }
    do
    {
        LCD_EN  = 1;
        _lcddelay();
        lcddata = LCD_DATA;
        LCD_EN  = 0;
    } while (lcddata & 0x90);

}

void _lcdwrite(unsigned char wdata, unsigned char di)
{
    _lcdwait();
    LCD_DI = di;        //
    LCD_RW = 0;        //
    LCD_DATA = wdata;
    LCD_EN = 1;         // strobe
    _lcddelay();
    LCD_EN = 0;
}

unsigned char _lcdread(void)
{


unsigned char lcddata;
    LCD_DI = 1;
    LCD_RW = 1;
    LCD_DATA = 0xFF;
    LCD_EN  = 1;
    _lcddelay();
    lcddata = LCD_DATA;
    LCD_EN  = 0;
return lcddata;

//    _asm
//        setb    LCD_DI
//
//    _endasm;

}




/*    
*****************************************************************************************
*********************** Atraso em milisegundos usando timer 0 ***************************
*/
void Atraso(unsigned char ms)
{
    tmp = ms;
    _asm
    0$: mov th0,#0xFC
        mov tl0,#0x17
        setb tr0
    1$: jnb tf0,1$
        clr tr0
        clr tf0
        djnz _tmp,0$
     _endasm;
}
/*    
*****************************************************************************************
************************** Atraso em segundos usando timer0 *****************************
*/
void DelSeg(unsigned char secs)
{
    tmp = secs;
    tmp = tmp * 20;

    _asm
    0$:
        mov th0,#0x3C
        mov tl0,#0xAF
        setb tr0
    1$: jnb tf0,1$
        clr tr0
        clr tf0
        djnz _tmp,0$
     _endasm;
}









